import os
from typing import Optional

import requests

from .exc import BotmakerException, InvalidAuth
from .helpers import sanitize_phone_number
from .resources import Message, Resource, TemplateMessage


class Client:
    BASE_URL = 'https://go.botmaker.com/api/v1.0'

    messages = Message
    template_messages = TemplateMessage

    def __init__(self, access_token: Optional[str] = None):
        self.session = requests.Session()
        self.access_token = access_token or os.environ['BOTMAKER_ACCESS_TOKEN']
        self.headers = {'access-token': self.access_token}
        Resource._client = self

    def get(self, endpoint: str, **kwargs) -> dict:
        return self.request('get', endpoint, {}, **kwargs)

    def post(self, endpoint: str, data: dict, **kwargs) -> dict:
        return self.request('post', endpoint, data, **kwargs)

    def request(
        self, method: str, endpoint: str, data: dict, **kwargs
    ) -> dict:
        url = self.BASE_URL + endpoint
        response = self.session.request(
            method, url, headers=self.headers, json=data, **kwargs
        )
        self._check_response(response)
        return response.json()

    @staticmethod
    def _check_response(response):
        if response.ok:
            body = response.json()
            if body.get('problems'):
                raise BotmakerException(body['problems']['message'])
        if response.status_code == 401:
            raise InvalidAuth
        else:
            response.raise_for_status()

    def check_whatsapp_contact(
        self, channel: str, phone_number: str
    ) -> Optional[str]:
        """
        Check a single phone number.
        """
        result = self.check_whatsapp_contacts(channel, [phone_number])
        try:
            checked = result[phone_number]
        except KeyError:
            checked = None
        return checked

    def check_whatsapp_contacts(
        self, channel: str, phone_numbers: list
    ) -> dict:
        """
        Check a list of phone numbers.
        Based on
        https://botmakeradmin.github.io/docs/es/#/messages-api?id=chequear-validez-de-n%C3%BAmeros-de-contactos-de-whatsapp
        """
        channel = sanitize_phone_number(channel)
        data = dict(chatChannelNumber=channel, contacts=phone_numbers)
        resp = self.post('/customer/checkWhatsAppContact', data)
        try:
            result = resp['result']
        except KeyError:
            # This should never happen
            raise BotmakerException("Expected 'result' in the response body")
        return result



import pytest

from botmaker import Client


@pytest.fixture(scope='module')
def vcr_config():
    config = dict()
    # Replace the Authorization request header with 'DUMMY' in cassettes
    config['filter_headers'] = [('access-token', 'DUMMY')]
    config['filter_post_data_parameters'] = [
        ('chatChannelNumber', '5215500000000'),
        ('platformContactId', '5215522222222'),
        ('contacts', ['+55 1 55 1234 5678']),
    ]
    return config


@pytest.fixture
def client():
    # Using BOTMAKER_ACCESS_TOKEN from env
    yield Client()



from dataclasses import dataclass, field

from .base import Resource


@dataclass
class TemplateMessage(Resource):
    _endpoint = '/intent/v2'

    id: str
    from_: str
    to: str
    template: str
    params: dict = field(default_factory=dict)

    @classmethod
    def create(
        cls,
        from_: str,
        to: str,
        template: str,
        chat_platform: str = 'whatsapp',
        check_phone: bool = True,
        **params,
    ):
        """
        Based on
        https://botmakeradmin.github.io/docs/es/#/messages-api?id=templates-messages
        """
        return cls.commonCreate(
            from_,
            to,
            chat_platform,
            check_phone,
            template=template,
            params=params,
        )



from dataclasses import dataclass

from .base import Resource


@dataclass
class Message(Resource):
    _endpoint = '/message/v3'

    id: str
    from_: str
    to: str
    message_text: str

    @classmethod
    def create(
        cls,
        from_: str,
        to: str,
        message_text: str,
        check_phone: bool = False,
        chat_platform: str = 'whatsapp',
    ):
        """
        Based on
        https://botmakeradmin.github.io/docs/es/#/messages-api?id=enviando-mensajes-a-los-usuarios
        """
        return cls.commonCreate(
            from_, to, chat_platform, check_phone, message_text=message_text
        )



import pytest

from botmaker import Client
from botmaker.exc import InvalidAuth


@pytest.mark.vcr
def test_invalid_auth():
    invalid_access_token = 'xxx'
    client = Client(invalid_access_token)
    with pytest.raises(InvalidAuth):
        client.get('/auth/credentials')


@pytest.mark.vcr
def test_valid_auth(client):
    resp = client.get('/auth/credentials')
    expected_resp_key = {'accessToken', 'clientId', 'refreshToken', 'secretId'}
    assert set(resp.keys()) == expected_resp_key



import pytest

from botmaker.exc import InvalidPhoneNumber


@pytest.mark.vcr
def test_template_message(client):
    tm = client.template_messages.create(
        '5215500000000',
        '+55 1 55 1234 5678',
        'phone_number_verification',
        codigo_de_6_digitos='123456',
    )
    assert tm.id
    assert tm == tm
    assert tm.to == '5515512345678'


@pytest.mark.vcr
def test_invalid_whatsapp_number(client):
    with pytest.raises(InvalidPhoneNumber):
        client.template_messages.create(
            '5215500000000',
            '123',
            'phone_number_verification',
            codigo_de_6_digitos='123456',
        )



from typing import ClassVar

from botmaker.exc import InvalidPhoneNumber
from botmaker.helpers import sanitize_phone_number


class Resource:
    _client: ClassVar['botmaker.Client']  # type: ignore # noqa: F821
    _endpoint: str

    def __init__(self, *_, **__):
        ...

    @classmethod
    def commonCreate(
        cls, from_: str, to: str, chat_platform: str, check_phone: bool, **data
    ):
        from_ = sanitize_phone_number(from_)
        if chat_platform == 'whatsapp' and check_phone:
            checked = cls._client.check_whatsapp_contact(from_, to)
            if not checked:
                raise InvalidPhoneNumber(
                    f"'{to} is not from valid WhatsApp contact"
                )
            else:
                to = checked
        else:
            to = sanitize_phone_number(to)
        body = dict(
            chatPlatform=chat_platform,
            chatChannelNumber=from_,
            platformContactId=to,
        )

        if cls._endpoint == '/message/v3':
            body['messageText'] = data['message_text']
            resp = cls._client.post(cls._endpoint, body)
            return cls(resp['id'], from_, to, data['message_text'])
        if cls._endpoint == '/intent/v2':
            body['ruleNameOrId'] = data['template']
            body['params'] = data['params']
            resp = cls._client.post(cls._endpoint, body)
            return cls(resp['id'], from_, to, data['template'], data['params'])



import pytest

from botmaker.exc import BotmakerException


@pytest.mark.vcr
def test_check_whatsapp_contact(client):
    phone_number = '+55 1 55 1234 5678'
    checked_contact = '5515512345678'
    assert checked_contact == client.check_whatsapp_contact(
        '5215500000000', phone_number
    )


@pytest.mark.vcr
def test_invalid_whatsapp_contact(client):
    assert client.check_whatsapp_contact('5215500000000', '123') is None


@pytest.mark.vcr
def test_check_whatsapp_contacts(client):
    contacts = ['+55 1 55 1234 5678', '123']
    result = client.check_whatsapp_contacts('5215500000000', contacts)
    assert '+55 1 55 1234 5678' in result  # whatsapp
    assert result['+55 1 55 1234 5678'] == '5515512345678'
    assert '123' not in result  # no whatsapp


@pytest.mark.vcr
def test_invalid_channel(client):
    with pytest.raises(BotmakerException):
        client.check_whatsapp_contacts('52 55 1234 5678', ['123'])



import pytest

from botmaker.exc import InvalidPhoneNumber


@pytest.mark.vcr
def test_message(client):
    tm = client.messages.create(
        '5215500000000',
        '+55 1 55 1234 5678',
        message_text='message test',
    )
    assert tm.id
    assert tm == tm
    assert repr(tm)
    assert str(tm)
    assert tm.to == '5515512345678'


@pytest.mark.vcr
def test_invalid_whatsapp_number_message(client):
    with pytest.raises(InvalidPhoneNumber):
        client.messages.create(
            '5215500000000',
            '123',
            check_phone=True,
            message_text='message test',
        )



from typing import Optional

from requests import HTTPError


class BotmakerException(Exception):
    """Generic BotMaker API exception"""

    def __init__(self, message: Optional[str] = None):
        if message:
            self.message = message

    def __str__(self):
        return getattr(self, 'message', self.__class__.__doc__)


class InvalidAuth(BotmakerException, HTTPError):
    """Invalid API authentication credentials"""


class InvalidPhoneNumber(BotmakerException):
    """Phone number is not associated to a WhatsApp account"""



"""
    A client library for BotMaker (https://www.botmaker.com/)

    :copyright: © 2019 Cuenca.
    :license: MIT, see LICENSE for more details.
"""
__all__ = ['Client', '__version__']

from .client import Client
from .version import __version__



from importlib.machinery import SourceFileLoader

import setuptools

install_requirements = [
    'requests>=2.21.0,<3.0.0',
    'dataclasses>=0.6;python_version<"3.7"',
]

version = SourceFileLoader('version', 'botmaker/version.py').load_module()


test_requires = [
    'pytest',
    'pytest-vcr',
    'pytest-cov',
    'black',
    'isort',
    'flake8==3.7.9',
    'mypy==0.812',
]

with open('README.md', 'r') as f:
    long_description = f.read()


setuptools.setup(
    name='botmaker',
    version=version.__version__,
    author='Cuenca',
    author_email='dev@cuenca.com',
    description='BotMaker API Client',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/cuenca-mx/botmaker-python',
    packages=setuptools.find_packages(),
    python_requires='>=3.6',
    install_requires=install_requirements,
    setup_requires=['pytest-runner'],
    tests_require=test_requires,
    extras_require=dict(test=test_requires),
    classifiers=[
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
)
